#--------------------------------------------------------------
# EMBL CBNA 
# Ontologies - statistics, biases, tools, networks, and interpretation October 2020 
# script v.0.5
# Matt Rogon Centre for Biomolecular Network Analysis EMBL
# rogon@embl.de
# 
# Ontologies (gene, disease) and Reactome Pathways for enrichment analysis
#
# This script is provided under the 
# Creative Commons - Attribution 4.0 international license
# you are free to Share and Adapt und the following conditions
# https://creativecommons.org/licenses/by/4.0/
#
#--------------------------------------------------------------

# install packages and load libraries
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install()
BiocManager::install(c("GenomicFeatures", "AnnotationDbi", "STRINGdb", "clusterProfiler", "pathview", "org.Mm.eg.db", "org.Hs.eg.db", "biomaRt", "igraph"))
install.packages(c("ggupset","dplyr", "plyr", "reshape", "reshape2", "devtools", "tidyverse", "httr", "jsonlite", "RJSONIO"))
devtools::install_github(c("GuangchuangYu/doseplot"))
BiocManager::install("enrichplot")
BiocManager::install("ReactomePA")
BiocManager::install("org.Mm.eg.db")
BiocManager::install("topGO")

# subset of developer versions only if the main repository is down or packages fail due to bugs
# do not install these unless specifically advised
#devtools::install_github(c("GuangchuangYu/DOSE", "GuangchuangYu/clusterProfiler", "GuangchuangYu/ReactomePA", "GuangchuangYu/doseplot"))
#devtools::install_github("IOR-Bioinformatics/PCSF", repos=BiocInstaller::biocinstallRepos(), dependencies=TRUE, type="source", force=TRUE)

# load libraries
#library(RJSONIO)
#library(STRINGdb)

library(dplyr)
library(reshape)
library(reshape2)
library(tidyr)
library(tidyverse)
library(httr)
library(igraph)
library(ReactomePA)
library(doseplot)
library(enrichplot)
library(devtools)
library(clusterProfiler)
library(pathview)
library(org.Hs.eg.db)
library(org.Mm.eg.db)
library(DOSE)


#--------------------------------------------------------------
# Load R Dataset
#--------------------------------------------------------------
# set your working directory
setwd("~/Part 2 - R-code session ClusterProfiler, ReactomePA, pathfindR/1. Load data and script")

# load the RData file:
load("Complete_2020.RData")

# some sample expression data:
GeneList <- read.table("selected_genes.txt", quote="\"", comment.char="", stringsAsFactors=FALSE)
GeneList <- unique(GeneList)

library(DOSE)
# expression values:
data(geneList)
de <- names(geneList)[geneList > 1]

# extract entrez id's from the annotation frame 'results' - this frame comes from BioMart
list_Entrez <- as.list(results$entrezgene)

#---------------------------------------------------------------------------------------
# ClusterProfiler - KEGG enrichment
# enrich against the whole of KEGG
# for a custom background we can use the universe switch
#---------------------------------------------------------------------------------------

??clusterProfiler

# which is simply a list of entrez id's same as the list_Entrez for input genes
kk <- enrichKEGG(list_Entrez, organism="human", pvalueCutoff=0.05, pAdjustMethod="BH", qvalueCutoff=0.1)
head(summary(kk))
as.data.frame(kk)


# Extract networks from KEGG into iGraph
hsa03430 <- pathview(gene.data = as.character(list_Entrez), pathway.id = "hsa03430",species = "human", limit = list(gene = max(abs(geneList)),cpd = 1))
hsa03030 <- pathview(gene.data = as.character(list_Entrez), pathway.id = "hsa03030",species = "human", limit = list(gene = max(abs(geneList)),cpd = 1))

#---------------------------------------------------------------------------------------
# GO
# let's repeat for gene ontology
#---------------------------------------------------------------------------------------
kg <- enrichGO(list_Entrez, OrgDb = "org.Hs.eg.db", ont="BP", pvalueCutoff=0.05, pAdjustMethod="BH", qvalueCutoff=0.1, minGSSize = 5)
head(summary(kg))
summary(kg)


# now some visualisations:
# cnetplot depicts the associations of genes with biological concepts 
# i.e. gene to term (GO/pathway)
cnetplot(kk)

# relationships between terms
emapplot(kk)

# The upsetplot is an alternative to cnetplot for visualizing the complex association 
# between genes and gene sets. It emphasizes the gene overlapping among different gene
# sets.
upsetplot(kk)
heatplot(kk)
dotplot(kk)
barplot(kk, showCategory = 15)
dotplot(kg)
barplot(kg, showCategory = 15)

write.table(kk, file="kegg_on_hits.txt", row.names = F, quote = FALSE, sep="\t")
write.table(kg, file="go_on_hits.txt", row.names = F, quote = FALSE, sep="\t")


# custom background option can be found under the universe switch for the enrichment functions

result <- summary(kk)
write.table(result, file="KEGG_enrichment_2020.txt", row.names = F, quote = FALSE, sep="\t")

# export entrez ID's for the hit list (will be used for filtering in Cytoscape)
lapply(list_Entrez, write, "entrez_list_hits_2020.txt", append=TRUE, ncolumns=1000)



#---------------------------------------------------------------------------------------
# Disease Ontologies uses the DOSE package
# let's find disease ontology enrichment
# notice the background
#---------------------------------------------------------------------------------------
library(DOSE)
genes <- unlist(list_Entrez)
kd <- enrichDO(gene = list_Entrez, ont = "DO", pvalueCutoff = 0.05, pAdjustMethod = "BH", universe = names(list_Entrez), minGSSize = 5, maxGSSize = 500, qvalueCutoff = 0.05, readable = FALSE)
head(kd)
cnetplot(kd)

# NCG Network of Cancer Gene
k_ncg <- enrichNCG(genes)
head(k_ncg)

# disgenet
k_dgn <- enrichDGN(genes)
head(k_dgn)
cnetplot(k_dgn)

# convert entrez id's to readable gene names for the enrichment result
k_dgn_readable <- setReadable(k_dgn, 'org.Hs.eg.db', 'ENTREZID')
cnetplot(k_dgn_readable)
upsetplot(k_dgn_readable)

## categorySize can be scaled by 'pvalue' or 'geneNum'
cnetplot(k_dgn_readable, categorySize="pvalue", foldChange=geneList)

# with layouts
cnetplot(k_dgn_readable, foldChange=geneList, circular = TRUE, colorEdge = TRUE)



#---------------------------------------------------------------------------------------
# Reactome Pathways
# Explore ReactomePA package and using the above command set perform enrichment of the 
# same list of identifiers against reactome
#---------------------------------------------------------------------------------------

library(ReactomePA)
kr <- enrichPathway(list_Entrez, organism="human", pvalueCutoff=0.05, pAdjustMethod="BH", qvalueCutoff=0.1, readable=T)
head(kr)
result_kr <- as.data.frame(kr)
write.table(result_kr, file="Reactome_enrichment_2019.txt", row.names = F, quote = FALSE, sep="\t")
cnetplot(kr)






#---------------------------------------------------------------------------------------
#
# Breakout Room 1 - exercises
#
#---------------------------------------------------------------------------------------
# Exercise 1/5
#---------------------------------------------------------------------------------------

# Using the dataframe: results
# Perform an enrichment against KEGG Module
# use the proper function for that purpose
# .....
#




# How many modules did you find?
# .....
#



#---------------------------------------------------------------------------------------
# Exercise 2
#---------------------------------------------------------------------------------------
# set working directory to the subdirectory 
# /Exercise 2

# Given a list of hits and a background in folder /Exercise 2
# background.txt and id_list.txt

# perform enrichment analysis against 
# 1. Gene Ontology,
# 2. Kegg modules, and
# 3. DisGeNet (diseases)

# For each result produce a cnetplot and an upsetplot.
# - you will need to provide the correct id's for the functions to execute - use converters
# - vary the p-value q-value thresholds in case of an empty result
# - make the disgenet output readable (gene names, not entrez id's)
# .....
#



# you will need to convert identifiers - for that purpose I suggest using the biomaRt package in R, alternatively you can use the website itselt
# ensembl biomart http://www.ensembl.org/biomart/martview/
# .....
#

library("biomaRt")

# part 1 Gene ontology

# part 2  MKEGG

# part 3 DisGeNet






#---------------------------------------------------------------------------------------
# Exercise 3 Comparing clusters/conditions/cells etc.
#---------------------------------------------------------------------------------------
# set working directory to the subdirectory 
# /Exercise 3
# For this exercise we will use the 550prot_clusters.txt file from the Exercise 3 directory


# load the 550prot_clusters.txt file into R
# in this file you have UniProt IDs and cluster numbers


# You will need to convert uniprot to entrez for ClusterProfiler
# you can use BioMart or bitr function in clusterProfiler




#--------------------------------------------------------------
# Enrichment with ClusterProfiler
#--------------------------------------------------------------
# to do enrichment on a 'per cluster' basis you will need the cluster column from the data frame:
# this will become your input to the ClusterProfiler function for comparing clusters.


# find the function in ClusterProfiler which will do the comparison and look up the format
# which it requires to work with.

# you can produce a single list per cluster and assemble all into a list of lists

#----
# Perform the Multi-cluster analysis against
#----
# 1. Gene Ontology, 
# 2. Reactome, and 
# 3. KEGG comparison










#---------------------------------------------------------------------------------------
# Exercise 4 Comparing clusters and building a custom annotation
#---------------------------------------------------------------------------------------
# set working directory to the subdirectory 
# /Exercise 4

# here you will find 2 files:
# cluster1.txt
# and
# cluster2.txt
# all_gene_disease_associations.tsv



#----------------------------------------------------------------------------------------
# TODO:
# - perform comparative cluster analysis for the two clusters 
# 	against the custom annotation file found in all_gene_disease_associations.tsv
# 
#----------------------------------------------------------------------------------------
# - explore the function compareCluster and enricher to complete the task
# 	compareCluster accepts 'fun=' switch, which works the same aspassing
# 	fun='enrichGO' or fun='enrichKEGG' i.e. you can pass fun ='enricher'
#----------------------------------------------------------------------------------------
# 1. load data - cluster 1 and cluster 2 into individual hit lists 
# 
# 2. the first column contains gene symbols - convert them to entrez id's
# 	here is an example on how to proceed with bitr function for annotation: 
# 	keytypes(org.Hs.eg.db)
# 	hit_ids <- bitr(hit_list$gene, fromType="SYMBOL", toType=c("ENTREZID"), OrgDb="org.Hs.eg.db")
#
# 3. create a list of lists which you will pass to compareCluster


# 3. Create a custom annotation 
# 	load annotation
# 	create a custom annotation for use with enricher for that we will use the 
# 	all_gene_disease_associations.tsv file and create two dataframes from this annotation:
# 		1. disease2gene
# 		2. disease2name

# 	the first maps disease id to gene id
# 	the second maps disease id to disease names
# 	as you may imagine you will need the gene id to be entrez for both the hit list and the annotation file
# 	first column of the source annotation file is the entrezID


# Once the datasets are constructed you will use the compareCluster function to
# run enricher with the above created annotations

# produce dotplot comparing the two clusters







#---------------------------------------------------------------------------------------
# Exercise 5 PathFindR + Tutorial (3 exercises below)
#---------------------------------------------------------------------------------------
# Using quantitative measurements and network data for pathway enrichment

# set your working directory to /Exercise 5 pathfindr
library(pathfindR)

# openVignette(pathfindR)
# Let's explore the package and functions
# https://github.com/egeulgen/pathfindR
# paper
# https://www.frontiersin.org/articles/10.3389/fgene.2019.00858/full
# website
# https://egeulgen.github.io/pathfindR/

# load sample data into a dataframe
input_df <- pathfindR.data::RA_input


# have a look at the formatting
View(input_df)

# all you need are 3 columns to execute the pathfindR main wrapper
output_df <- run_pathfindR(input_df)

# visualise the output_df results
# Enriched Terms Data Frame
# run_pathfindR() returns a data frame of enriched terms. Columns are:
#    ID: ID of the enriched term
#    Term_Description: Description of the enriched term
#    Fold_Enrichment: Fold enrichment value for the enriched term (Calculated using ONLY the input genes)
#    occurrence: The number of iterations that the given term was found to enriched over all iterations
#    lowest_p: the lowest adjusted-p value of the given term over all iterations
#    highest_p: the highest adjusted-p value of the given term over all iterations
#    non_Signif_Snw_Genes (OPTIONAL): the non-significant active subnetwork genes, comma-separated (controlled by list_active_snw_genes, default is FALSE)
#    Up_regulated: the up-regulated genes (as determined by change value > 0, if the change column was provided) in the input involved in the given term’s gene set, comma-separated. If change column was not provided, all affected input genes are listed here.
#    Down_regulated: the down-regulated genes (as determined by change value < 0, if the change column was provided) in the input involved in the given term’s gene set, comma-separated
#
# The first 2 rows of the output data frame of the example analysis on the rheumatoid arthritis gene-level differential expression input data (RA_input) is shown below:

knitr::kable(head(output_df, 2))

enrichment_chart(output_df)
term_gene_heatmap(result_df = output_df, genes_df = input_df)
term_gene_graph(result_df = output_df, use_description = TRUE)
UpSet_plot(result_df = output_df, genes_df = input_df)

# cluster 
output_df_clustered_fuzzy <- cluster_enriched_terms(output_df, method = "fuzzy")



#--------------------------------------------------------------------------------------
# Exercise 1:
# Load the following file: 
# pathfinder_input_exercise_clusters.tsv
#
# prepare the dataframe for pathfindeR
# you will need 3 columns:
# Gene name
# logFC
# adj P Val
# Run pathfindR wrapper function on this dataset
# visualise



#--------------------------------------------------------------------------------------
# Exercise 2
# Create a custom script that will execute pathfindR with the following settings:
# Benjamini-Hochberg FDR
# adjusted p_val_threshold 0.01
# gene sets: GO-BP
# minimum gene set size 5, maximum 100
# PIN protein interaction reference network: IntAct
# 
# generate a fuzzy-clustered result network of terms
# term gene heatmap
# term gene graph
# upset plot

#--------------------------------------------------------------------------------------

# Exercise 3
# Use the original file pathfinder_input_exercise_clusters.txt and compare clusters 1 and 2 with pathfindR
# using combine_pathfindR_results function
# produce visualisations

#--------------------------------------------------------------------------------------
